"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNft = exports.createMint = exports.createMintInstructions = exports.createAtaAndMint = exports.createAta = exports.createAtaAndMintInstructions = exports.createAtaAndTransfer = exports.createAtaAndTransferInstructions = exports.mintTo = void 0;
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
function mintTo(provider, mint, amount, destination) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintTx = new web3_js_1.Transaction();
        mintTx.add((0, spl_token_1.createMintToInstruction)(mint, destination, provider.wallet.publicKey, amount));
        try {
            yield provider.sendAndConfirm(mintTx);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
    });
}
exports.mintTo = mintTo;
function createAtaAndTransferInstructions(provider, mint, amount, from = provider.wallet.publicKey, to = provider.wallet.publicKey, payer = provider.wallet.publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const toAta = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, to, true);
        const instructions = [];
        if (!(yield provider.connection.getAccountInfo(toAta))) {
            instructions.push((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, toAta, to, mint));
        }
        const fromAta = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, from, true);
        if (amount != 0) {
            instructions.push((0, spl_token_1.createTransferInstruction)(fromAta, toAta, from, BigInt(amount.toString())));
        }
        return {
            instructions,
            toAta,
        };
    });
}
exports.createAtaAndTransferInstructions = createAtaAndTransferInstructions;
function createAtaAndTransfer(provider, mint, amount, from = provider.wallet.publicKey, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const transferIx = new web3_js_1.Transaction();
        const { instructions, toAta } = yield createAtaAndTransferInstructions(provider, mint, amount, from, to, payer);
        if (instructions.length > 0)
            transferIx.add(...instructions);
        try {
            if (instructions.length > 0)
                yield provider.sendAndConfirm(transferIx, undefined, confirmOptions);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
        return toAta;
    });
}
exports.createAtaAndTransfer = createAtaAndTransfer;
function createAtaAndMintInstructions(provider, mint, amount, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const ata = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, to, true);
        const instructions = [];
        if (!(yield provider.connection.getAccountInfo(ata))) {
            instructions.push((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, ata, to, mint));
        }
        if (amount != 0) {
            instructions.push((0, spl_token_1.createMintToInstruction)(mint, ata, authority, BigInt(amount.toString())));
        }
        return {
            instructions,
            ata,
        };
    });
}
exports.createAtaAndMintInstructions = createAtaAndMintInstructions;
function createAta(provider, mint, owner = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const tx = new web3_js_1.Transaction();
        const ata = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, owner, true);
        if (!(yield provider.connection.getAccountInfo(ata))) {
            tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, ata, owner, mint));
        }
        try {
            yield provider.sendAndConfirm(tx, undefined, confirmOptions);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
        return ata;
    });
}
exports.createAta = createAta;
function createAtaAndMint(provider, mint, amount, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintTx = new web3_js_1.Transaction();
        const { instructions, ata } = yield createAtaAndMintInstructions(provider, mint, amount, to, authority, payer);
        if (instructions.length > 0)
            mintTx.add(...instructions);
        try {
            if (instructions.length > 0)
                yield provider.sendAndConfirm(mintTx, undefined, confirmOptions);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
        return ata;
    });
}
exports.createAtaAndMint = createAtaAndMint;
function createMintInstructions(provider, decimals, mintAuthority, freezeAuthority = null, mintKeypair = web3_js_1.Keypair.generate()) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintKey = mintKeypair.publicKey;
        return [
            web3_js_1.SystemProgram.createAccount({
                fromPubkey: provider.wallet.publicKey,
                newAccountPubkey: mintKey,
                space: 82,
                lamports: yield provider.connection.getMinimumBalanceForRentExemption(82),
                programId: spl_token_1.TOKEN_PROGRAM_ID,
            }),
            yield (0, spl_token_1.createInitializeMintInstruction)(mintKeypair.publicKey, decimals, mintAuthority, freezeAuthority),
        ];
    });
}
exports.createMintInstructions = createMintInstructions;
function createMint(provider, decimals, mintAuthority, freezeAuthority = null, mintKeypair = web3_js_1.Keypair.generate()) {
    return __awaiter(this, void 0, void 0, function* () {
        const tx = new web3_js_1.Transaction();
        tx.add(...(yield createMintInstructions(provider, decimals, mintAuthority, freezeAuthority, mintKeypair)));
        try {
            yield provider.sendAndConfirm(tx, [mintKeypair]);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
        return mintKeypair.publicKey;
    });
}
exports.createMint = createMint;
function createNft(provider, recipient, data = {}, collectionKey, mintKeypair = web3_js_1.Keypair.generate(), holderKey = provider.wallet.publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintKey = mintKeypair.publicKey;
        const instructions = yield createMintInstructions(provider, 0, provider.wallet.publicKey, provider.wallet.publicKey, mintKeypair);
        const [metadata] = web3_js_1.PublicKey.findProgramAddressSync([
            Buffer.from("metadata", "utf8"),
            mpl_token_metadata_1.PROGRAM_ID.toBuffer(),
            mintKey.toBuffer(),
        ], mpl_token_metadata_1.PROGRAM_ID);
        instructions.push(yield (0, mpl_token_metadata_1.createCreateMetadataAccountV3Instruction)({
            metadata,
            mint: mintKey,
            mintAuthority: provider.wallet.publicKey,
            payer: provider.wallet.publicKey,
            updateAuthority: provider.wallet.publicKey,
        }, {
            createMetadataAccountArgsV3: {
                data: Object.assign({ name: "test", symbol: "TST", uri: "https://shdw-drive.genesysgo.net/6tcnBSybPG7piEDShBcrVtYJDPSvGrDbVvXmXKpzBvWP/dc.json", sellerFeeBasisPoints: 10, creators: [
                        {
                            address: holderKey,
                            verified: true,
                            share: 100,
                        },
                    ], collection: collectionKey
                        ? { key: collectionKey, verified: false }
                        : null, uses: null }, data),
                isMutable: true,
                collectionDetails: null,
            },
        }));
        const { instructions: mintInstrs } = yield createAtaAndMintInstructions(provider, mintKeypair.publicKey, 1, recipient);
        instructions.push(...mintInstrs);
        if (collectionKey) {
            const [collectionMetadataAccount] = web3_js_1.PublicKey.findProgramAddressSync([
                Buffer.from("metadata", "utf8"),
                mpl_token_metadata_1.PROGRAM_ID.toBuffer(),
                collectionKey.toBuffer(),
            ], mpl_token_metadata_1.PROGRAM_ID);
            const [collectionMasterEdition] = web3_js_1.PublicKey.findProgramAddressSync([
                Buffer.from("metadata", "utf8"),
                mpl_token_metadata_1.PROGRAM_ID.toBuffer(),
                collectionKey.toBuffer(),
                Buffer.from("edition", "utf8"),
            ], mpl_token_metadata_1.PROGRAM_ID);
            const instruction = (0, mpl_token_metadata_1.createVerifyCollectionInstruction)({
                metadata: metadata,
                collectionAuthority: provider.wallet.publicKey,
                payer: provider.wallet.publicKey,
                collectionMint: collectionKey,
                collection: collectionMetadataAccount,
                collectionMasterEditionAccount: collectionMasterEdition,
            });
            instructions.push(instruction);
        }
        const [edition] = web3_js_1.PublicKey.findProgramAddressSync([
            Buffer.from("metadata", "utf8"),
            mpl_token_metadata_1.PROGRAM_ID.toBuffer(),
            mintKey.toBuffer(),
            Buffer.from("edition", "utf8"),
        ], mpl_token_metadata_1.PROGRAM_ID);
        instructions.push((0, mpl_token_metadata_1.createCreateMasterEditionV3Instruction)({
            edition,
            mint: mintKey,
            updateAuthority: provider.wallet.publicKey,
            mintAuthority: provider.wallet.publicKey,
            payer: provider.wallet.publicKey,
            metadata,
        }, {
            createMasterEditionArgs: {
                maxSupply: 0,
            },
        }));
        const tx = new web3_js_1.Transaction();
        tx.add(...instructions);
        try {
            yield provider.sendAndConfirm(tx, [mintKeypair]);
        }
        catch (e) {
            console.log("Error", e, e.logs);
            if (e.logs) {
                console.error(e.logs.join("\n"));
            }
            throw e;
        }
        return {
            mintKey,
            collectionKey,
        };
    });
}
exports.createNft = createNft;
//# sourceMappingURL=token.js.map